﻿var TodoList = TodoList || {};

TodoList.Priority = {
    VeryLow: 1,
    Low: 2,
    Normal: 3,
    High: 4,
    VeryHigh: 5
};

// Define TodoListSettings: the global object for application settings
var TodoListSettings = WinJS.Class.define(function () {
    var localSettings = Windows.Storage.ApplicationData.current.localSettings;
    var that = {};
    that.defaultPriority = TodoList.Priority.Normal;
    that.load = function () {
        _loadFromSettings();
    };
    that.save = function () {
        _saveToSettings();
    };

    function _loadFromSettings() {
        var priority = localSettings.values["defaultPriority"];
        if (priority) {
            that.defaultPriority = priority;
        }
    };

    function _saveToSettings() {
        localSettings.values["defaultPriority"] = that.defaultPriority;
    };

    return that;
});
TodoList.settings = new TodoListSettings();

// Define the Task object
var Task = WinJS.Class.define(function () {
    var that = {};
    that.description = "This is my new task";
    that.dueDate = TodoList.firstOfNextMonth();
    that.priority = TodoList.Priority.Normal;
    that.status = "Not Started";
    that.percCompleted = 0;
    that.minPriority = TodoList.Priority.VeryLow;
    that.maxPriority = TodoList.Priority.VeryHigh;

    if (TodoList.settings.defaultPriority != "undefined")
        that.priority = TodoList.settings.defaultPriority;
    return that;
});



// App initializer
TodoList.init = function () {
    // Register handler for resize events
    window.onresize = addEventListener('resize', TodoList.onResize, false);

    // Register handler for buttons
    document.getElementById("buttonNewTask").addEventListener("click", TodoList.newTaskClick);
    document.getElementById("buttonOpenTask").addEventListener("click", TodoList.openTaskClick);
    document.getElementById("buttonAddTask").addEventListener("click", TodoList.addTaskClick);
    document.getElementById("buttonShare").addEventListener("click", TodoList.shareClick);

    // Initialization of Share source contract
    var clipboard = Windows.ApplicationModel.DataTransfer.DataTransferManager.getForCurrentView();
    clipboard.addEventListener("datarequested", function (e) {
        var currentTask = TodoList.getTaskFromUI();

        if (currentTask.description.length === 0) {
            e.request.failWithDisplayText("You need to indicate a description of the task.");
            return;
        }
        TodoList.shareDataAsHtml(e, currentTask);
        TodoList.shareDataAsPlainText(e, currentTask);
    });

    // Load settings and initialize view
    TodoList.settings.load();
    TodoList.displayTask(new Task());

    var s = "2012-12-01T11:00:00.000Z";
    var d = new Date(s);
    TodoList.alert(d.toLocaleDateString());
}

TodoList.shareDataAsPlainText = function (e, task) {
    var request = e.request;
    request.data.properties.title = "TO DO";
    request.data.properties.description = task.description;
    request.data.setText(task.description + " due by " + task.dueDate.toLocaleDateString());
}

TodoList.shareDataAsHtml = function (e, task) {
    var request = e.request;
    request.data.properties.title = "TO DO";
    request.data.properties.description = task.description;

    // Load the HTML layout (and run data binding)
    var elem = document.getElementById("shareHtml");
    WinJS.Binding.processAll(elem, task);
    var rawHtml = elem.innerHTML;

    // Make the raw HTML compliant with Windows 8 requirements
    var html = Windows.ApplicationModel.DataTransfer.HtmlFormatHelper.createHtmlFormat(rawHtml);
    request.data.setHtmlFormat(html);

    // This extra work is necessary ONLY if the HTML references an image.

    // This is the URL of the image as in the HTML block. You transform it in a URI object
    // and store it as an in-memory stream in the DataPackage resources.
    var localImage = "ms-appx:///images/todolist-icon.png";
    var url = new Windows.Foundation.Uri(localImage);
    var streamRef = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(url);
    request.data.resourceMap[localImage] = streamRef;
}

TodoList.addTaskClick = function () {
    var currentTask = TodoList.getTaskFromUI();
    if (!TodoList.validateInput(currentTask)) {
        TodoList.alert("There is something wrong with the data you entered.");
        return;
    }

    TodoList.displaySummary(currentTask);
}

TodoList.newTaskClick = function () {
    TodoList.displayTask(new Task);
}

TodoList.openTaskClick = function () {
    TodoList.pickFileAndOpenTask();
}

TodoList.shareClick = function () {
    Windows.ApplicationModel.DataTransfer.DataTransferManager.showShareUI();
};

TodoList.alert = function (message) {
    var alertDialog = new Windows.UI.Popups.MessageDialog(message);
    alertDialog.showAsync();
}

TodoList.displayTask = function (task) {
    TodoList.performInitialBinding(task);
}

TodoList.performInitialBinding = function (task) {

    // Enable binding on the HTML element(s) of choice 
    var bindableElement = document.getElementById("form-container");
    WinJS.Binding.processAll(bindableElement, task);

    var datePicker = document.getElementById("taskDueDate").winControl;
    datePicker.current = task.dueDate;
    var select = document.getElementById("taskStatus");
    select.selectedIndex = TodoList.getIndexFromStatus(task.status);
}

TodoList.firstOfNextMonth = function () {
    var d = new Date();
    d.setDate(d.getDate() + 31);
    var year = d.getFullYear();
    var month = d.getMonth();
    var day = 1;
    var newDate = new Date(year, month, day);
    return newDate;
}

TodoList.getIndexFromStatus = function (option) {
    switch (option) {
        case "In progress": return 1;
        case "Completed": return 2;
        default: return 0;
    }
}

TodoList.getTaskFromUI = function () {
    var task = new Task();

    var taskSubject = document.getElementById("taskSubject");
    task.description = taskSubject.value;

    var taskDueDate = document.getElementById("taskDueDate").winControl;
    task.dueDate = taskDueDate.current;

    var taskPriority = document.getElementById("taskPriority");
    task.priority = taskPriority.value;

    var taskStatus = document.getElementById("taskStatus");
    task.status = taskStatus.options[taskStatus.selectedIndex].value;

    var taskPercCompleted = document.getElementById("taskPercCompleted");
    task.percCompleted = taskPercCompleted.value;

    return task;
}

TodoList.validateInput = function (task) {
    // Check description is NOT empty
    if (task.description.length == 0)
        return false;

    // Check date is a future date
    if (task.dueDate <= new Date())
        return false;

    // Check if perc-completed is >0 and task not started or 100 done but not completed
    if ((task.percCompleted > 0 && task.status == "Not Started")  ||
        (task.percCompleted == 100 && task.status != "Completed") ||
        (task.percCompleted != 100 && task.status == "Completed"))
        return false;

    return true;
}

TodoList.displaySummary = function (task) {
    // Prepare the content for the flyout
    var bindableElement = document.getElementById("flyoutSummary");
    WinJS.Binding.processAll(bindableElement, task);

    // Display the flyout
    var anchor = document.getElementById("buttonAddTask");
    var flyoutSummary = document.getElementById("flyoutSummary").winControl;
    flyoutSummary.show(anchor);
}

TodoList.onResize = function (e) {
    var snappedLayout = document.getElementById("list-of-tasks");
    var defaultLayout = document.getElementById("task-editor");
    var viewState = Windows.UI.ViewManagement.ApplicationViewState.value;
}

TodoList.pickFileAndSaveTask = function () {
    var currentState = Windows.UI.ViewManagement.ApplicationView.value;
    if (currentState === Windows.UI.ViewManagement.ApplicationViewState.snapped &&
        !Windows.UI.ViewManagement.ApplicationView.tryUnsnap()) {
        // Fail silently if we can't unsnap
        return;
    }
    
    // Get the task object to save
    var currentTask = TodoList.getTaskFromUI();

    // Create the picker object and set options
    TodoList.invokeSavePicker(currentTask);
    //TodoList.invokeFolderPicker(currentTask);
}

TodoList.invokeSavePicker = function (task) {
    var savePicker = new Windows.Storage.Pickers.FileSavePicker();
    savePicker.suggestedStartLocation = Windows.Storage.Pickers.PickerLocationId.computerFolder;
    savePicker.fileTypeChoices.insert("TodoList Task", [".todo"]);
    savePicker.commitButtonText = "Create task";
    savePicker.suggestedFileName = task.description;

    // Invoke the file picker
    savePicker.pickSaveFileAsync().then(function (file) {
        if (file) {
            // The file ALREADY exists; the file picker CREATED it with 0 bytes
            Windows.Storage.FileIO
                .writeTextAsync(file, TodoList.serializeTask(task))
                .done(function () {
                            TodoList.alert("Data successfully saved");
                        },
                        function (error) {
                            TodoList.alert("Unable to save data. Sorry about that!");
                        }
                    );
        }
    });
}

TodoList.pickFileAndOpenTask = function () {
    var currentState = Windows.UI.ViewManagement.ApplicationView.value;
    if (currentState === Windows.UI.ViewManagement.ApplicationViewState.snapped &&
        !Windows.UI.ViewManagement.ApplicationView.tryUnsnap()) {
        // Fail silently if we can't unsnap
        return;
    }

    TodoList.invokeOpenPicker();
}

TodoList.invokeOpenPicker = function () {
    var openPicker = new Windows.Storage.Pickers.FileOpenPicker();
    openPicker.suggestedStartLocation = Windows.Storage.Pickers.PickerLocationId.computerFolder;
    openPicker.fileTypeFilter.replaceAll([".todo"]);
    openPicker.pickSingleFileAsync().then(function (file) {
        if (file) {
            var io = Windows.Storage.FileIO;
            io.readTextAsync(file)
              .done(function (json) {
                        var task = JSON.parse(json);
                        task.dueDate = new Date(task.dueDate);
                        TodoList.displayTask(task);
                      },
                    function () { TodoList.alert("Unable to read the file") });
        }
    });
}

// Task Serializer
TodoList.serializeTask = function (task) {
    //return task.description + "|" +
    //    task.dueDate + "|" + 
    //    task.priority + "|" + 
    //    task.status + "|" + 
    //    task.percCompleted;
    return JSON.stringify(task);
}

// This is used only for DEMO purposes
TodoList.invokeFolderPicker = function (task) {
    var folderPicker = new Windows.Storage.Pickers.FolderPicker();
    folderPicker.fileTypeFilter.replaceAll(["*"]);
    folderPicker.SuggestedStartLocation = Windows.Storage.Pickers.PickerLocationId.computerFolder;
    folderPicker.SettingsIdentifier = "FolderPicker";

    // Invoke the folder picker
    var fileOptions = Windows.Storage.CreationCollisionOption;
    var io = Windows.Storage.FileIO;
    folderPicker
        .pickSingleFolderAsync()
        .done(
            function (folder) {
                if (folder) {
                    folder.createFileAsync("sample.todo", fileOptions.replaceExisting)
                          .done(function (file) {
                                    io.writeTextAsync(file, "Some more data.")
                                      .then(function () { io.appendTextAsync(file, " By me."); })
                                      .done(function () { TodoList.alert("All done!")})
                                },
                                function (error) {
                                    TodoList.alert("Unable to create file. Sorry about that!");
                                });
                }
            });
}


// Converters
TodoList.dateForDisplay = WinJS.Binding.converter(function (value) {
    return value.toLocaleDateString();
});
TodoList.priorityForDisplay = WinJS.Binding.converter(function (value) {
    switch (value) {
        case TodoList.Priority.VeryLow:
            return "Very low";
        case TodoList.Priority.VeryHigh:
            return "Very high";
        case TodoList.Priority.Low:
            return "Low";
        case TodoList.Priority.High:
            return "High";
        default:
            return "Normal";
    }
});
TodoList.percForDisplay = WinJS.Binding.converter(function (value) {
    return value + "%";
});