﻿var TodoList = TodoList || {};

TodoList.init = function () {
    document.getElementById("buttonAddTask").addEventListener("click", TodoList.addTaskClick);
    TodoList.performInitialBinding()
}

TodoList.addTaskClick = function () {
    var currentTask = TodoList.getTaskFromUI();
    if (!TodoList.validateInput(currentTask)) {
        TodoList.alert("There is something wrong with the data you entered.");
        return;
    }

    TodoList.displaySummary(currentTask);
}

TodoList.alert = function (message) {
    var alertDialog = new Windows.UI.Popups.MessageDialog(message);
    alertDialog.showAsync();
}

TodoList.Priority = {
    VeryLow: 1,
    Low: 2,
    Normal: 3,
    High: 4,
    VeryHigh: 5
};

// Define the Task object
var Task = WinJS.Class.define(function () {
    var that = {};
    that.description = "This is my new task";
    that.dueDate = TodoList.firstOfNextMonth();
    that.priority = TodoList.Priority.Normal;
    that.status = "Not Started";
    that.percCompleted = 0;
    that.minPriority = TodoList.Priority.VeryLow;
    that.maxPriority = TodoList.Priority.VeryHigh;
    return that;
});

TodoList.performInitialBinding = function () {
    var task = new Task();

    // Enable binding on the HTML element(s) of choice 
    var bindableElement = document.getElementById("form-container");
    WinJS.Binding.processAll(bindableElement, task);

    var datePicker = document.getElementById("taskDueDate").winControl;
    datePicker.current = task.dueDate;
    var select = document.getElementById("taskStatus");
    select.selectedIndex = TodoList.getIndexFromStatus(task.status); 
}

TodoList.firstOfNextMonth = function () {
    var d = new Date();
    d.setDate(d.getDate() + 31);
    var year = d.getFullYear();
    var month = d.getMonth();
    var day = 1;
    var newDate = new Date(year, month, day);
    return newDate;
}

TodoList.getIndexFromStatus = function (option) {
    switch (option) {
        case "In progress": return 1;
        case "Completed": return 2;
        default: return 0;
    }
}

TodoList.getTaskFromUI = function () {
    var task = new Task();

    var taskSubject = document.getElementById("taskSubject");
    task.description = taskSubject.value;

    var taskDueDate = document.getElementById("taskDueDate").winControl;
    task.dueDate = taskDueDate.current;

    var taskPriority = document.getElementById("taskPriority");
    task.priority = taskPriority.value;

    var taskStatus = document.getElementById("taskStatus");
    task.status = taskStatus.options[taskStatus.selectedIndex].value;

    var taskPercCompleted = document.getElementById("taskPercCompleted");
    task.percCompleted = taskPercCompleted.value;

    return task;
}

TodoList.validateInput = function (task) {
    // Check description is NOT empty
    if (task.description.length == 0)
        return false;

    // Check date is a future date
    if (task.dueDate <= new Date())
        return false;

    // Check if perc-completed is >0 and task not started or 100 done but not completed
    if ((task.percCompleted > 0 && task.status == "Not Started")  ||
        (task.percCompleted == 100 && task.status != "Completed") ||
        (task.percCompleted != 100 && task.status == "Completed"))
        return false;

    return true;
}

TodoList.displaySummary = function (task) {
    var description = "<p><span>DESCRIPTION</span>: " + task.description + "<p>";
    var dueDate = "<p><span>DUE DATE</span>: " + task.dueDate + "<p>";
    var priority = "<p><span>PRIORITY</span>: " + task.priority + "<p>";
    var status = "<p><span>STATUS</span>: " + task.status + "<p>";
    var percCompleted = "<p><span>% COMPLETED</span>: " + task.percCompleted + "<p>";
    var summary = description + dueDate + priority + status + percCompleted;

    // Prepare the content for the flyout
    document.getElementById("flyoutSummary").innerHTML = summary;  

    var anchor = document.getElementById("buttonAddTask");
    var flyoutSummary = document.getElementById("flyoutSummary").winControl;
    flyoutSummary.show(anchor);
}